@@ -8,7 +8,7 @@ import (
 	"golang.org/x/sys/unix"
 )
 
-func kernelArch() (string, error) {
+func KernelArch() (string, error) {
 	var utsname unix.Utsname
 	err := unix.Uname(&utsname)
 	return string(utsname.Machine[:bytes.IndexByte(utsname.Machine[:], 0)]), err
