@@ -37,6 +37,12 @@ func AvgWithContext(ctx context.Context) (*AvgStat, error) {
 	return ret, nil
 }
 
+type forkstat struct {
+	forks    int
+	vforks   int
+	__tforks int
+}
+
 // Misc returns miscellaneous host-wide statistics.
 // darwin use ps command to get process running/blocked count.
 // Almost same as Darwin implementation, but state is different.
@@ -64,5 +70,11 @@ func MiscWithContext(ctx context.Context) (*MiscStat, error) {
 		}
 	}
 
+	f, err := getForkStat()
+	if err != nil {
+		return nil, err
+	}
+	ret.ProcsCreated = f.forks
+
 	return &ret, nil
 }
